' Visual Basic.NET interface for Motherboard Monitor 5
' by Pedro Navarro (pnavarro@telematicarc.net)
'
' v1.0 - 11/19/2003 - Initial Release. Code somewhat inefficient because I haven't found a way to marshal arrays of user defined types
'
' -------------------------------------------------------------------------------
' 
' Based in code by Christian Hofstaedlter and http://www.codeproject.com
'
' -------------------------------------------------------------------------------
'

Imports System.Runtime.InteropServices

'Bus types enumeration
Public Enum mbmBusType
    btISA = 0
    btSMBus = 1
    btVIA686ABus = 2
    btDirectIO = 3
End Enum

'SMBus types enumeration
Public Enum mbmSMBType
    smtSMBIntel = 0
    smtSMBAMD = 1
    smtSMBALi = 2
    smtSMBNForce = 3
    smtSMBSIS = 4
End Enum

'Sensor types enumeration
Public Enum mbmSensorType
    stUnknown = 0
    stTemperature = 1
    stVoltage = 2
    stFan = 3
    stMhz = 4
    stPercentage = 5
End Enum

<StructLayout(LayoutKind.Sequential)> Public Class mbmSharedInfo
    Public siSMB_Base As Short      'SMBus base address
    Private m_siSMB_Type As Byte    'SMBus/Isa bus used to access chip
    Private m_siSMB_Code As Byte    'SMBus sub type, Intel, AMD or ALi
    Public siSMB_Addr As Byte       'Address of sensor chip on SMBus
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=41)> Private m_siSMB_Name As Char() 'Nice name for SMBus
    Public siISA_Base As Short      'ISA base address of sensor chip on ISA
    Public siChipType As Integer    'Chip nr, connects with Chipinfo.ini
    Public siVoltageSubType As Byte 'Subvoltage option selected

    Public ReadOnly Property siSMB_Type() As mbmBusType
        Get
            siSMB_Type = m_siSMB_Type
        End Get
    End Property

    Public ReadOnly Property siSMB_Code() As mbmSMBType
        Get
            siSMB_Code = m_siSMB_Code
        End Get
    End Property

    Public ReadOnly Property siSmb_Name() As String
        Get
            siSmb_Name = CStr(m_siSMB_Name).TrimEnd(Chr(0))
        End Get
    End Property
End Class

'Index object. Contains the number of sensors of a type
<StructLayout(LayoutKind.Sequential)> Public Class mbmSharedIndex
    Private m_iType As Byte 'Sensor type
    Public Count As Integer 'Number of sensors of that type

    Public ReadOnly Property iType() As mbmSensorType
        Get
            iType = m_iType
        End Get
    End Property
End Class

'Sensor object. Contains the information about a sensor
<StructLayout(LayoutKind.Sequential)> Public Class mbmSharedSensor
    Private m_ssType As Byte 'Sensor type
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=12)> Private m_ssName As Char() 'Sensor name
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=3)> Public sspadding1 As Char() 'Padding
    Public ssCurrent As Double 'Current Value
    Public ssLow As Double
    Public ssHigh As Double
    Public ssCount As Integer ': LongInt;                          // total number of readout
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=4)> Public ssPad2 As Byte() ': array [0..3] of Char              // padding of 4 byte
    Public ssTotal As Double ': Extended;                         // total amout of all readouts
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=6)> Public ssPad3 As Char() ': array [0..5] of Char              // padding of 6 byte
    Public ssAlarm1 As Double ': Double;                           // temp & fan: low alarm; voltage: % off;
    Public ssAlarm2 As Double ': Double;                           // temp: high alarm

    Public ReadOnly Property ssType() As mbmSensorType
        Get
            ssType = m_ssType
        End Get
    End Property

    Public ReadOnly Property ssName() As String
        Get
            ssName = CStr(m_ssName).TrimEnd(Chr(0))
        End Get
    End Property

End Class

<StructLayout(LayoutKind.Sequential)> Public Class mbmSharedData
    Public Version As Double

    'IMPORTANT NOTE: As of version 5.3.5.0 there are 10 indices, each one 8 bytes long
    'the number 80 below means 8 * 10. In case MBM's author changes either the number
    'of indices or their size, you will have to change that number also.
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=80)> Private IndicesInfo As Char()

    'IMPORTANT NOTE: As of version 5.3.5.0 there are 100 sensors, each one 80 bytes long
    'the number 8000 below means 80 * 100. In case MBM's author changes either the number
    'of sensors or their size, you will have to change that number also.
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=8000)> Private SensorsInfo As Char()

    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=56)> Private m_sdInfo As Char()
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=41)> Private m_sdStart As Char()
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=41)> Private m_sdCurrent As Char()
    <MarshalAs(UnmanagedType.ByValArray, SizeConst:=256)> Private m_sdPath As Char()

    'Returns a specified sensor object
    Public Function Sensor(ByVal SensorNumber As Integer) As mbmSharedSensor
        Dim newSensor As New mbmSharedSensor

        Return CopyStringToClass(Mid(CStr(SensorsInfo), (Marshal.SizeOf(newSensor) * (SensorNumber - 1)) + 1, Marshal.SizeOf(newSensor)), newSensor)
    End Function

    'Returns a specified index object
    Public Function Index(ByVal IndexNumber As Integer) As mbmSharedIndex
        Dim newIndex As New mbmSharedIndex

        Return CopyStringToClass(Mid(CStr(IndicesInfo), (Marshal.SizeOf(newIndex) * (IndexNumber - 1)) + 1, Marshal.SizeOf(newIndex)), newIndex)
    End Function

    Public Function sdInfo() As mbmSharedInfo
        Dim newSharedInfo As New mbmSharedInfo

        Return CopyStringToClass(m_sdInfo, newSharedInfo)
    End Function

    Public ReadOnly Property sdStart() As String
        Get
            sdStart = CStr(m_sdStart).TrimEnd(Chr(0))
        End Get
    End Property

    Public ReadOnly Property sdCurrent() As String
        Get
            sdCurrent = CStr(m_sdCurrent).TrimEnd(Chr(0))
        End Get
    End Property

    Public ReadOnly Property sdPath() As String
        Get
            sdPath = CStr(m_sdPath).TrimEnd(Chr(0))
        End Get
    End Property


    '*********************************************************************************************
    '*  API Declarations to open the shared memory and to copy the contained information ...
    '*********************************************************************************************
    Private Declare Function OpenFileMapping Lib "kernel32" Alias "OpenFileMappingA" (ByVal dwDesiredAccess As Integer, ByVal bInheritHandle As Integer, ByVal lpName As String) As Integer
    Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Integer) As Integer
    Private Declare Function MapViewOfFile Lib "kernel32" (ByVal hFileMappingObject As Integer, ByVal dwDesiredAccess As Integer, ByVal dwFileOffsetHigh As Integer, ByVal dwFileOffsetLow As Integer, ByVal dwNumberOfBytesToMap As Integer) As Integer
    Private Declare Function UnmapViewOfFile Lib "kernel32" (ByVal lpBaseAddress As Integer) As Integer
    Private Const FILE_MAP_WRITE As Short = &H2S
    Private Const FILE_MAP_READ As Short = &H4S
    Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (ByVal pDst As IntPtr, ByVal pSrc As String, ByVal ByteLen As Long)

    ' copies a string into a class' properties using CopyMemory
    Private Function CopyStringToClass(ByVal Source As String, ByVal Target As Object) As Object

        If Target Is Nothing Then Return Nothing

        Dim p_objTarget As IntPtr

        p_objTarget = Marshal.AllocHGlobal(Marshal.SizeOf(Target))

        CopyMemory(p_objTarget, Source, Marshal.SizeOf(Target))
        Marshal.PtrToStructure(p_objTarget, Target)

        Marshal.FreeHGlobal(p_objTarget)

        Return Target

    End Function   'CopyStringToClass 

    'Refreshes MBM data
    Public Function Refresh(Optional ByRef bSilent As Boolean = True)
        Dim myMBMFile As Integer
        Dim myMBMMem As Integer

        'Open the memory mapped file
        myMBMFile = OpenFileMapping(FILE_MAP_READ, False, "$M$B$M$5$S$D$")

        'If the file is not located exit the function returning Nothing
        If myMBMFile = 0 Then
            If (bSilent) Then
                Return Nothing
            Else
                MsgBox("MBM Data File/Mem could not be opened. Sorry.")
                Return Nothing
            End If
        End If

        'Get the pointer to MBM's shared memory
        myMBMMem = MapViewOfFile(myMBMFile, FILE_MAP_READ, 0, 0, 0)

        'Convert that value to an IntPtr
        Dim baseAddress As IntPtr
        baseAddress = IntPtr.op_Explicit(myMBMMem)

        'Create the object where MBM's shared memory will be copied
        'Dim newSharedData As New TSharedData

        'Copy from memory to our object
        Marshal.PtrToStructure(baseAddress, Me)

        'Close Handles
        UnmapViewOfFile(myMBMMem)
        CloseHandle(myMBMFile)

        'Mission accomplished

    End Function


End Class
